# Copyright (c) 2020 Arda Seremet <ardaseremet@outlook.com>

import aiohttp
import async_timeout
from asyncio import TimeoutError

class API:
    """Class to interact with the API of ProgettiHWSW boards."""

    def __init__(self, ip: str):
        """Initialize the API."""
        self.ip = ip

    async def request(self, path: str):
        try:
            with async_timeout.timeout(5):
                async with aiohttp.request("GET", f"{self.ip}/{path}") as resp:
                    return await resp.text()
        except TimeoutError:
            return False
    
    async def execute(self, code: int):
        """Make requests with API codes for boards."""
        try:
            return await self.request(f"index.htm?execute={code}")
        except Exception:
            return False



# Copyright (c) 2020 Arda Seremet <ardaseremet@outlook.com>

from lxml import etree

from ProgettiHWSW.api import API
from ProgettiHWSW.input import Input
from ProgettiHWSW.relay import Relay

from .const import STATUS_XML_PATH


class ProgettiHWSWAPI:
    """Class to communicate with ProgettiHWSW boards."""

    def __init__(self, ip: str, is_old_board: bool = False):
        """Initialize the API and return the corresponding object class."""
        self.api = API(f"http://{ip}")
        self.ip = ip
        self.is_old_board = is_old_board

    async def check_board(self):
        """Check if this board is existing."""
        request = await self.api.request(STATUS_XML_PATH)
        if request is False:
            return False

        root = etree.XML(request)

        is_old_board = len(root.xpath("//led0")) > 0
        relay_tags = root.xpath("//*[starts-with(local-name(), 'led')]")
        input_tags = root.xpath("//*[starts-with(local-name(), 'btn')]")
        self.is_old_board = is_old_board

        if len(relay_tags) <= 0 and len(input_tags) <= 0:
            return False

        return {
            "title": f"{len(relay_tags)}R & {len(input_tags)}IN Board",
            "is_old": is_old_board,
            "relay_count": len(relay_tags),
            "input_count": len(input_tags),
        }

    async def get_switches(self):
        """Return all switch states."""
        request = await self.api.request(STATUS_XML_PATH)
        if request is False:
            return False

        root = etree.XML(request)
        relay_tags = root.xpath("//*[starts-with(local-name(), 'led')]")
        if len(relay_tags) <= 0:
            return False
        is_old_board = len(root.xpath("//led0")) > 0
        self.is_old_board = is_old_board

        states = {}
        for i in relay_tags:
            number = int(i.tag.split("led")[1])
            states[number if is_old_board is False else number + 1] = (
                True if i.text == "1" else False
            )
        return states

    async def get_inputs(self):
        """Return all switch states."""
        request = await self.api.request(STATUS_XML_PATH)
        if request is False:
            return False

        root = etree.XML(request)
        relay_tags = root.xpath("//*[starts-with(local-name(), 'btn')]")
        if len(relay_tags) <= 0:
            return False
        is_old_board = len(root.xpath("//btn0")) > 0
        self.is_old_board = is_old_board

        states = {}
        for i in relay_tags:
            number = int(i.tag.split("btn")[1])
            states[number if is_old_board is False else number + 1] = (
                True if i.text == "up" else False
            )
        return states

    def get_relay(self, relay_number: int, relay_mode: str = "bistable") -> Relay:
        """Return the Relay class."""
        return Relay(self.api, relay_number, relay_mode, self.is_old_board)

    def get_input(self, input_number: int) -> Input:
        """Return the Input class."""
        return Input(self.api, input_number, self.is_old_board)



# Copyright (c) 2020 Arda Seremet <ardaseremet@outlook.com>

from lxml import etree

from ProgettiHWSW.api import API
from ProgettiHWSW.const import STATUS_XML_PATH


class Input:
    """Class that represents an input object."""

    def __init__(self, api: API, input_number: int, is_old: bool):
        """Initialize Input class."""
        self.input_number = int(input_number)
        self.api = api
        self.state = None
        self.is_old = is_old

    @property
    def id(self) -> int:
        """Return the input number."""
        return self.input_number

    @property
    def is_on(self) -> bool:
        """Return if the input is on."""
        return self.state

    async def update(self):
        """Update the input status."""
        request = await self.api.request(STATUS_XML_PATH)
        if request is False:
            return False

        root = etree.XML(request)
        number = self.input_number - 1 if self.is_old is True else self.input_number
        if not number >= 0:
            return False

        path = root.xpath(f"//btn{str(number)}")
        if not len(path) > 0:
            return False

        self.state = True if path[0].text == "up" else False
        return True



# Copyright (c) 2020 Arda Seremet <ardaseremet@outlook.com>

from lxml import etree

from ProgettiHWSW.api import API
from ProgettiHWSW.const import (
    STATUS_XML_PATH,
    TEMP_MONOSTABLE_BASE,
    TOGGLE_BASE,
    TURN_OFF_BASE,
    TURN_ON_BASE,
)


class Relay:
    """Clas that represents a relay object."""

    def __init__(self, api: API, relay_number: int, relay_mode: str, is_old: bool):
        """Initialize Relay class."""
        self.relay_number = int(relay_number)
        self.relay_mode = relay_mode
        self.api = api
        self.state = None
        self.is_old = is_old

    @property
    def id(self) -> int:
        """Return the relay number."""
        return self.relay_number

    @property
    def is_on(self) -> bool:
        """Return if the relay is on."""
        return self.state

    async def toggle(self):
        """Toggle the relay."""
        command = TOGGLE_BASE + self.relay_number
        await self.api.execute(command)

    async def control(self, state: bool):
        """Control the relay state."""
        command = (
            (TURN_ON_BASE if self.relay_mode == "bistable" else TEMP_MONOSTABLE_BASE)
            if state is True
            else TURN_OFF_BASE
        ) + self.relay_number
        await self.api.execute(command)

    async def update(self):
        """Update the relay status."""
        request = await self.api.request(STATUS_XML_PATH)
        if request is False:
            return False

        root = etree.XML(request)
        number = self.relay_number - 1 if self.is_old is True else self.relay_number
        if not number >= 0:
            return False

        path = root.xpath(f"//led{str(number)}")
        if not len(path) > 0:
            return False

        self.state = True if path[0].text == "1" else False
        return True



import os
import sys

import setuptools


def read_file(file):
    with open(os.path.join(os.path.dirname(__file__), file)) as readable:
        return readable.read()


README = read_file("README.md")

setuptools.setup(
    name="ProgettiHWSW",
    version="0.1.1",
    long_description="\n\n".join([README]),
    long_description_content_type="text/markdown",
    description="Controls ProgettiHWSW relay boards.",
    url="http://github.com/ardaseremet/progettihwsw",
    download_url="http://github.com/ardaseremet/progettihwsw/tarball/0.1.1",
    author="Arda Seremet",
    author_email="ardaseremet@outlook.com",
    license="MIT",
    packages=setuptools.find_packages(),
    install_requires=["aiohttp", "lxml"],
    zip_safe=False,
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Intended Audience :: Telecommunications Industry",
        "Intended Audience :: Information Technology",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Legal Industry",
    ],
)
